@@ -4,16 +4,16 @@ use strict;
 use warnings;
 use bytes;
 
-use IO::Compress::Base::Common 2.048 qw(createSelfTiedObject);
+use IO::Compress::Base::Common 2.055 qw(createSelfTiedObject);
 
-use IO::Uncompress::Base 2.048 ;
+use IO::Uncompress::Base 2.055 ;
 
 
 require Exporter ;
 
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, $AnyUncompressError);
 
-$VERSION = '2.048';
+$VERSION = '2.055';
 $AnyUncompressError = '';
 
 @ISA = qw( Exporter IO::Uncompress::Base );
@@ -27,22 +27,22 @@ Exporter::export_ok_tags('all');
 
 BEGIN
 {
-   eval ' use IO::Uncompress::Adapter::Inflate 2.048 ;';
-   eval ' use IO::Uncompress::Adapter::Bunzip2 2.048 ;';
-   eval ' use IO::Uncompress::Adapter::LZO 2.048 ;';
-   eval ' use IO::Uncompress::Adapter::Lzf 2.048 ;';
-   eval ' use IO::Uncompress::Adapter::UnLzma 2.048 ;';
-   eval ' use IO::Uncompress::Adapter::UnXz 2.048 ;';
+   eval ' use IO::Uncompress::Adapter::Inflate 2.055 ;';
+   eval ' use IO::Uncompress::Adapter::Bunzip2 2.055 ;';
+   eval ' use IO::Uncompress::Adapter::LZO 2.055 ;';
+   eval ' use IO::Uncompress::Adapter::Lzf 2.055 ;';
+   eval ' use IO::Uncompress::Adapter::UnLzma 2.055 ;';
+   eval ' use IO::Uncompress::Adapter::UnXz 2.055 ;';
 
-   eval ' use IO::Uncompress::Bunzip2 2.048 ;';
-   eval ' use IO::Uncompress::UnLzop 2.048 ;';
-   eval ' use IO::Uncompress::Gunzip 2.048 ;';
-   eval ' use IO::Uncompress::Inflate 2.048 ;';
-   eval ' use IO::Uncompress::RawInflate 2.048 ;';
-   eval ' use IO::Uncompress::Unzip 2.048 ;';
-   eval ' use IO::Uncompress::UnLzf 2.048 ;';
-   eval ' use IO::Uncompress::UnLzma 2.048 ;';
-   eval ' use IO::Uncompress::UnXz 2.048 ;';
+   eval ' use IO::Uncompress::Bunzip2 2.055 ;';
+   eval ' use IO::Uncompress::UnLzop 2.055 ;';
+   eval ' use IO::Uncompress::Gunzip 2.055 ;';
+   eval ' use IO::Uncompress::Inflate 2.055 ;';
+   eval ' use IO::Uncompress::RawInflate 2.055 ;';
+   eval ' use IO::Uncompress::Unzip 2.055 ;';
+   eval ' use IO::Uncompress::UnLzf 2.055 ;';
+   eval ' use IO::Uncompress::UnLzma 2.055 ;';
+   eval ' use IO::Uncompress::UnXz 2.055 ;';
 }
 
 sub new
@@ -60,7 +60,7 @@ sub anyuncompress
 
 sub getExtraParams
 {
-    use IO::Compress::Base::Common 2.048 qw(:Parse);
+    use IO::Compress::Base::Common 2.055 qw(:Parse);
     return ( 'RawInflate' => [1, 1, Parse_boolean,  0] ,
              'UnLzma'     => [1, 1, Parse_boolean,  0] ) ;
 }
@@ -616,7 +616,7 @@ The string '-' can be used as an alias for standard input.
 =item A scalar reference 
 
 If C<$input> is a scalar reference, the compressed data will be read from
-C<$$output>.
+C<$$input>.
 
 =back
 
