@@ -42,11 +42,11 @@ char *instr(char *big, const char *little);
 STR *
 walk(int useval, int level, register int node, int *numericptr, int minprec)
 {
-    register int len;
-    register STR *str;
-    register int type;
-    register int i;
-    register STR *tmpstr;
+    int len;
+    STR *str;
+    int type;
+    int i;
+    STR *tmpstr;
     STR *tmp2str;
     STR *tmp3str;
     char *t;
@@ -69,7 +69,7 @@ walk(int useval, int level, register int node, int *numericptr, int minprec)
 	if (namelist) {
 	    while (isALPHA(*namelist)) {
 		for (d = tokenbuf,s=namelist;
-		  isALPHA(*s) || isDIGIT(*s) || *s == '_';
+		  isWORDCHAR(*s);
 		  *d++ = *s++) ;
 		*d = '\0';
 		while (*s && !isALPHA(*s)) s++;
@@ -84,11 +84,6 @@ walk(int useval, int level, register int node, int *numericptr, int minprec)
 	str = walk(0,level,ops[node+1].ival,&numarg,P_MIN);
 	if (do_split && need_entire && !absmaxfld)
 	    split_to_array = TRUE;
-	if (do_split && split_to_array)
-	    set_array_base = TRUE;
-	if (set_array_base) {
-	    str_cat(str,"$[ = 1;\t\t\t# set array base to 1\n");
-	}
 	if (fswitch && !const_FS)
 	    const_FS = fswitch;
 	if (saw_FS > 1 || saw_RS)
@@ -589,7 +584,7 @@ sub Pick {\n\
 		    *t &= 127;
 		    if (isLOWER(*t))
 			*t = toUPPER(*t);
-		    if (!isALPHA(*t) && !isDIGIT(*t))
+		    if (!isALNUMC(*t))
 			*t = '_';
 		}
 		if (!strchr(tokenbuf,'_'))
@@ -650,9 +645,10 @@ sub Pick {\n\
 	str_set(str,"substr(");
 	str_scat(str,fstr=walk(1,level,ops[node+1].ival,&numarg,P_COMMA+1));
 	str_free(fstr);
-	str_cat(str,", ");
+	str_cat(str,", (");
 	str_scat(str,fstr=walk(1,level,ops[node+2].ival,&numarg,P_COMMA+1));
 	str_free(fstr);
+	str_cat(str,")-1");
 	if (len == 3) {
 	    str_cat(str,", ");
 	    str_scat(str,fstr=walk(1,level,ops[node+3].ival,&numarg,P_COMMA+1));
@@ -712,13 +708,13 @@ sub Pick {\n\
 	break;
     case OINDEX:
 	str = str_new(0);
-	str_set(str,"index(");
+	str_set(str,"(1+index(");
 	str_scat(str,fstr=walk(1,level,ops[node+1].ival,&numarg,P_COMMA+1));
 	str_free(fstr);
 	str_cat(str,", ");
 	str_scat(str,fstr=walk(1,level,ops[node+2].ival,&numarg,P_COMMA+1));
 	str_free(fstr);
-	str_cat(str,")");
+	str_cat(str,"))");
 	numeric = 1;
 	break;
     case OMATCH:
@@ -729,7 +725,7 @@ sub Pick {\n\
 	str_cat(str," =~ ");
 	str_scat(str,fstr=walk(1,level,ops[node+2].ival,&numarg,P_MATCH+1));
 	str_free(fstr);
-	str_cat(str," && ($RLENGTH = length($&), $RSTART = length($`)+1)");
+	str_cat(str," ? scalar($RLENGTH = length($&), $RSTART = length($`)+1) : 0");
 	numeric = 1;
 	break;
     case OUSERDEF:
@@ -970,12 +966,12 @@ sub Pick {\n\
 	    }
 	    else if (strEQ(str->str_ptr,"$NF")) {
 		numeric = 1;
-		str_set(str,"$#Fld");
+		str_set(str,"($#Fld+1)");
 	    }
 	    else if (strEQ(str->str_ptr,"$0"))
 		str_set(str,"$_");
 	    else if (strEQ(str->str_ptr,"$ARGC"))
-		str_set(str,"($#ARGV+1)");
+		str_set(str,"($#ARGV+2)");
 	}
 	else {
 #ifdef NOTDEF
@@ -988,7 +984,7 @@ sub Pick {\n\
 	    str_cat(tmpstr,"[]");
 	    tmp2str = hfetch(symtab,tmpstr->str_ptr);
 	    if (tmp2str && atoi(tmp2str->str_ptr))
-		str_cat(str,"[");
+		str_cat(str,"[(");
 	    else
 		str_cat(str,"{");
 	    str_scat(str,fstr=walk(1,level,ops[node+2].ival,&numarg,P_MIN));
@@ -999,7 +995,7 @@ sub Pick {\n\
 	    }
 	    else {
 		if (tmp2str && atoi(tmp2str->str_ptr))
-		    strcpy(tokenbuf,"]");
+		    strcpy(tokenbuf,")-1]");
 		else
 		    strcpy(tokenbuf,"}");
                *tokenbuf += (char)128;
@@ -1012,10 +1008,10 @@ sub Pick {\n\
 	str = str_new(0);
 	if (split_to_array) {
 	    str_set(str,"$Fld");
-	    str_cat(str,"[");
+	    str_cat(str,"[(");
 	    str_scat(str,fstr=walk(1,level,ops[node+1].ival,&numarg,P_MIN));
 	    str_free(fstr);
-	    str_cat(str,"]");
+	    str_cat(str,")-1]");
 	}
 	else {
 	    i = atoi(walk(1,level,ops[node+1].ival,&numarg,P_MIN)->str_ptr);
@@ -1116,7 +1112,7 @@ sub Pick {\n\
 		*t &= 127;
 		if (isLOWER(*t))
 		    *t = toUPPER(*t);
-		if (!isALPHA(*t) && !isDIGIT(*t))
+		if (!isALNUMC(*t))
 		    *t = '_';
 	    }
 	    if (!strchr(tokenbuf,'_'))
@@ -1153,7 +1149,7 @@ sub Pick {\n\
 		    *t &= 127;
 		    if (isLOWER(*t))
 			*t = toUPPER(*t);
-		    if (!isALPHA(*t) && !isDIGIT(*t))
+		    if (!isALNUMC(*t))
 			*t = '_';
 		}
 		if (!strchr(tokenbuf,'_'))
@@ -1424,7 +1420,7 @@ sub Pick {\n\
 	i = numarg;
 	if (i) {
 	    t = s = tmpstr->str_ptr;
-	    while (isALPHA(*t) || isDIGIT(*t) || *t == '$' || *t == '_')
+	    while (isWORDCHAR(*t) || *t == '$')
 		t++;
 	    i = t - s;
 	    if (i < 2)
@@ -1470,7 +1466,7 @@ sub Pick {\n\
 	tmp2str = hfetch(symtab,str->str_ptr);
 	if (tmp2str && atoi(tmp2str->str_ptr)) {
 	    sprintf(tokenbuf,
-	      "foreach %s ($[ .. $#%s) ",
+	      "foreach %s (0 .. $#%s) ",
 	      s,
 	      d+1);
 	}
@@ -1563,7 +1559,7 @@ tab(register STR *str, register int lvl)
 static void
 fixtab(register STR *str, register int lvl)
 {
-    register char *s;
+    char *s;
 
     /* strip trailing white space */
 
@@ -1581,7 +1577,7 @@ fixtab(register STR *str, register int lvl)
 static void
 addsemi(register STR *str)
 {
-    register char *s;
+    char *s;
 
     s = str->str_ptr+str->str_cur - 1;
     while (s >= str->str_ptr && (*s == ' ' || *s == '\t' || *s == '\n'))
@@ -1593,7 +1589,7 @@ addsemi(register STR *str)
 static void
 emit_split(register STR *str, int level)
 {
-    register int i;
+    int i;
 
     if (split_to_array)
 	str_cat(str,"@Fld");
@@ -1626,9 +1622,9 @@ emit_split(register STR *str, int level)
 int
 prewalk(int numit, int level, register int node, int *numericptr)
 {
-    register int len;
-    register int type;
-    register int i;
+    int len;
+    int type;
+    int i;
     int numarg;
     int numeric = FALSE;
     STR *tmpstr;
@@ -2043,8 +2039,8 @@ prewalk(int numit, int level, register int node, int *numericptr)
 static void
 numericize(register int node)
 {
-    register int len;
-    register int type;
+    int len;
+    int type;
     STR *tmpstr;
     STR *tmp2str;
     int numarg;
